/*
 Copyright The Helm Authors.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

package schema

type InputMessageTestV1 struct {
	Name string
}

type OutputMessageTestV1 struct {
	Greeting string
}

type ConfigTestV1 struct{}

func (c *ConfigTestV1) Validate() error {
	return nil
}
