// Code generated by "stringer -type=RESTScopeStrategy -linecomment"; DO NOT EDIT.

package watcher

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[RESTScopeAutomatic-0]
	_ = x[RESTScopeRoot-1]
	_ = x[RESTScopeNamespace-2]
}

const _RESTScopeStrategy_name = "automaticrootnamespace"

var _RESTScopeStrategy_index = [...]uint8{0, 9, 13, 22}

func (i RESTScopeStrategy) String() string {
	if i < 0 || i >= RESTScopeStrategy(len(_RESTScopeStrategy_index)-1) {
		return "RESTScopeStrategy(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _RESTScopeStrategy_name[_RESTScopeStrategy_index[i]:_RESTScopeStrategy_index[i+1]]
}
