/*
Copyright The cert-manager Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	acmev1 "github.com/cert-manager/cert-manager/pkg/client/applyconfigurations/acme/v1"
)

// IssuerSpecApplyConfiguration represents a declarative configuration of the IssuerSpec type for use
// with apply.
type IssuerSpecApplyConfiguration struct {
	IssuerConfigApplyConfiguration `json:",inline"`
}

// IssuerSpecApplyConfiguration constructs a declarative configuration of the IssuerSpec type for use with
// apply.
func IssuerSpec() *IssuerSpecApplyConfiguration {
	return &IssuerSpecApplyConfiguration{}
}

// WithACME sets the ACME field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ACME field is set to the value of the last call.
func (b *IssuerSpecApplyConfiguration) WithACME(value *acmev1.ACMEIssuerApplyConfiguration) *IssuerSpecApplyConfiguration {
	b.IssuerConfigApplyConfiguration.ACME = value
	return b
}

// WithCA sets the CA field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the CA field is set to the value of the last call.
func (b *IssuerSpecApplyConfiguration) WithCA(value *CAIssuerApplyConfiguration) *IssuerSpecApplyConfiguration {
	b.IssuerConfigApplyConfiguration.CA = value
	return b
}

// WithVault sets the Vault field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Vault field is set to the value of the last call.
func (b *IssuerSpecApplyConfiguration) WithVault(value *VaultIssuerApplyConfiguration) *IssuerSpecApplyConfiguration {
	b.IssuerConfigApplyConfiguration.Vault = value
	return b
}

// WithSelfSigned sets the SelfSigned field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the SelfSigned field is set to the value of the last call.
func (b *IssuerSpecApplyConfiguration) WithSelfSigned(value *SelfSignedIssuerApplyConfiguration) *IssuerSpecApplyConfiguration {
	b.IssuerConfigApplyConfiguration.SelfSigned = value
	return b
}

// WithVenafi sets the Venafi field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Venafi field is set to the value of the last call.
func (b *IssuerSpecApplyConfiguration) WithVenafi(value *VenafiIssuerApplyConfiguration) *IssuerSpecApplyConfiguration {
	b.IssuerConfigApplyConfiguration.Venafi = value
	return b
}
